/*
 * ADTPro - Apple Disk Transfer ProDOS
 * Copyright (C) 2007 by David Schmidt
 * david__schmidt at users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.adtpro.gui.AudioConfig;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.Mixer.Info;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.adtpro.gui.FrameUtils;
import org.adtpro.gui.GridBagUtil;
import org.adtpro.gui.Gui;
import org.adtpro.resources.Messages;
import org.adtpro.utilities.Log;
import org.adtpro.ADTProperties;

public abstract class Abstract extends JDialog implements ActionListener
{
  private static final long serialVersionUID = 1L;

  protected static org.adtpro.ADTProperties _properties = null;

  private JLabel labelAudioDevice;

  private JComboBox<String> comboAudioDevice;
  
  int _audioDeviceIndices[];

  public JButton okButton = new JButton(Messages.getString("Gui.Ok"));
  public JButton cancelButton = new JButton(Messages.getString("Gui.Cancel"));

  public static Gui _parent;
  
  public static final int CANCEL = 0, OK = 1;
  private int exitStatus = CANCEL;
  
  final private Mixer.Info[] mixerInfo;

  final private String className;
  
  public static void setPropertiesAndParent(ADTProperties properties, Gui parent) {
      _properties = properties;
      _parent = parent;
  }

  /**
   * getSingleton() is not synchronized, so we must check in this method to make
   * sure a concurrent getSingleton() didn't already allocate the Singleton
   * 
   * synchronized on a static method locks the class
   */
  protected synchronized static Abstract allocateSingleton()
  {
	  Abstract theSingleton = null;

	  final StackTraceElement [] s = new RuntimeException().getStackTrace();
	  final String className = s[1].getClassName();
	  try
	  {
		  theSingleton = (Abstract)Class.forName(className).newInstance();
	  }
	  catch (Exception e)
	  {
		  Log.println(false,className+".allocateSingleton() fatal error!" + e.toString());
		  e.printStackTrace();
	  }
	  return theSingleton;
  }

  /**
   * 
   * Protected constructor - use the <code>getSingleton</code> to instantiate.
   * 
   */
  protected Abstract()
  {
	this.className = this.getClass().getName().toString();
    this.setTitle(this.getDialogTitle());
    JPanel buttonPanel = new JPanel(new GridBagLayout());
    okButton.addActionListener(this);
    cancelButton.addActionListener(this);

    GridBagUtil.constrain(buttonPanel, okButton, 1, 1, // X, Y Coordinates
        1, 1, // Grid width, height
        GridBagConstraints.NONE, // Fill value
        GridBagConstraints.WEST, // Anchor value
        0.0, 0.0, // Weight X, Y
        5, 5, 5, 5); // Top, left, bottom, right insets
    GridBagUtil.constrain(buttonPanel, cancelButton, 2, 1, // X, Y Coordinates
        1, 1, // Grid width, height
        GridBagConstraints.NONE, // Fill value
        GridBagConstraints.WEST, // Anchor value
        0.0, 0.0, // Weight X, Y
        5, 5, 5, 5); // Top, left, bottom, right insets
    JPanel configPanel = new JPanel();
    configPanel.setLayout(new GridBagLayout());
    this.getContentPane().setLayout(new BorderLayout());    
    this.getContentPane().add(configPanel,BorderLayout.NORTH);
    this.getContentPane().add(buttonPanel,BorderLayout.SOUTH);
    Log.getSingleton();
    comboAudioDevice = new JComboBox<String>();
    comboAudioDevice.addItem(Messages.getString("Gui.DefaultAudioMixer"));
    
    this.mixerInfo = this.buildMixerInfo();
    
    String nextName = null;
    _audioDeviceIndices = new int[this.mixerInfo.length + 1];
    _audioDeviceIndices[0] = 0;
    int j = 1;
    for (int i = 0; i < this.mixerInfo.length; i++)
    {
      nextName = null;
      try
      {
        nextName = this.mixerInfo[i].getName();
        if (!nextName.equals("")) /* Skip it if it's name is blank... */
        {
          comboAudioDevice.addItem(nextName);
          _audioDeviceIndices[j] = i;
          Log.println(false, this.className+".ctor() Added device "+nextName+" at index "+i+", mixer index "+j+".");
          j = j + 1;
        }
      }
      catch (Exception e)
      {
        /* Don't need to see stack traces for bad lines/mixers... */
        /*
        Log.println(true,this.className+".ctor() Encountered error on mixer "+mixerInfo[i].getName()+":");
        Log.printStackTrace(e);
        */
      }
    }
/*
    Log.println(true, Messages.getString("Gui.NoAudio")); //$NON-NLS-1$
    Log.println(false, "AudioConfig Constructor could not instantiate the rxtx library.");
*/
    labelAudioDevice = new JLabel(this.getDialogTitle(), SwingConstants.LEFT); //$NON-NLS-1$

    GridBagUtil.constrain(configPanel, labelAudioDevice, 1, 1, // X, Y Coordinates
        1, 1, // Grid width, height
        GridBagConstraints.NONE, // Fill value
        GridBagConstraints.WEST, // Anchor value
        0.0, 0.0, // Weight X, Y
        5, 5, 0, 0); // Top, left, bottom, right insets
    GridBagUtil.constrain(configPanel, comboAudioDevice, 1, 2, // X, Y Coordinates
        1, 1, // Grid width, height
        GridBagConstraints.HORIZONTAL, // Fill value
        GridBagConstraints.WEST, // Anchor value
        0.0, 0.0, // Weight X, Y
        0, 5, 5, 5); // Top, left, bottom, right insets

    this.pack();
    this.setBounds(FrameUtils.center(this.getSize()));
    okButton.requestFocus();
    getRootPane().setDefaultButton(okButton);
    int mixerIndex = 0;
    try
    {
      mixerIndex = Integer.parseInt(_properties.getProperty(this.getPropertyNameForPort(),"0"));
      Log.println(false, this.className+".ctor() Found a GUI mixer index of: "+mixerIndex);
      Log.println(false, "  which is a hardware index of: "+_audioDeviceIndices[mixerIndex]);
    }
    catch (NumberFormatException e)
    {
      /* Leaves mixerIndex at zero */
    }
    comboAudioDevice.setSelectedIndex(mixerIndex);
    Log.println(false,"AudioConfig Constructor exit.");
  }

  public String getPort()
  {
    final Abstract theSingleton = this.getInstance();
    return (String)theSingleton.comboAudioDevice.getSelectedItem();
  }

  public void showDialog()
  {
	this.setModal(true);
    this.setBounds(FrameUtils.center(this.getSize(),_parent.getBounds()));
    this.setVisible(true);
  }

  public void actionPerformed(ActionEvent e)
  {
    final Abstract theSingleton = this.getInstance();
    Log.println(false,this.className+".actionPerformed() entry, responding to "+e.getActionCommand());
    if (e.getSource() == okButton)
    {
      Log.println(false, "  Selected index: "+(comboAudioDevice.getSelectedIndex()));
      if (comboAudioDevice.getSelectedIndex() > -1)
      {
        _properties.setProperty(
        		this.getPropertyNameForPort(), 
        		Integer.toString(comboAudioDevice.getSelectedIndex())
        	);
        Log.println(false, "  Set property to: "+Integer.toString(comboAudioDevice.getSelectedIndex()));
      }
      else
      {
        _properties.setProperty(this.getPropertyNameForPort(), "0");
        Log.println(false, "  Oops, set property to 0.");
      }
      _properties.save();
      theSingleton.exitStatus = OK;
      this.setVisible(false);
    }
    else if (e.getSource() == cancelButton)
    {
      int priorSelection = Integer.parseInt(_properties.getProperty(this.getPropertyNameForPort(), "0"));
      theSingleton.comboAudioDevice.setSelectedIndex(priorSelection); //$NON-NLS-1$ //$NON-NLS-2$
      theSingleton.exitStatus = CANCEL;
      this.setVisible(false);
    }
    Log.println(false,this.className+".actionPerformed() exit.");
  }

  public int getExitStatus()
  {
    return this.getInstance().exitStatus;
  }

  private Info[] buildMixerInfo() {
	  final Mixer.Info[] mixerInfo = AudioSystem.getMixerInfo();
	  final Vector<Mixer.Info> result = new Vector<Mixer.Info>();
	  String nextName = null;
	  int j = 0;
	  for (int i = 0; i < mixerInfo.length; i++)
	  {
		  nextName = null;
		  try
		  {
			  nextName = mixerInfo[i].getName();
			  if (!nextName.equals("") /* Skip it if it's name is blank... */
					  && this.isAcceptable(mixerInfo[i])
					  )
			  {
				  result.add(mixerInfo[i]);
				  Log.println(false, this.className+".buildMixedInfo() Added device "+nextName+" at index "+i+" into mixer index "+j+".");
			  }
		  }
		  catch (Exception e)
		  {
			  /* Don't need to see stack traces for bad lines/mixers... */
			  /*
	        Log.println(true,this.className+".buildMixerInfo() Encountered error on mixer "+mixerInfo[i].getName()+":");
	        Log.printStackTrace(e);
			   */
		  }
	  }
	  {
		  final Mixer.Info[] r = new Mixer.Info[result.size()]; 
		  return result.toArray(r);
	  }
  }
  
  public Info[] getMixerInfo() {
	return this.mixerInfo;
  }
  
	/*
	 * This method creates and returns an AudioFormat object for a given set of
	 * format parameters. The allowable parameter values are shown in comments
	 * following the declarations.
	 */
	public static AudioFormat getAudioFormat()
	{
		float sampleRate = 44100.0F; // 8000,11025,16000,22050,44100
		int sampleSizeInBits = 8; // 8,16
		int channels = 1; // 1,2
		boolean signed = true; // true,false
		boolean bigEndian = false; // true,false
		return new AudioFormat(sampleRate, sampleSizeInBits, channels, signed, bigEndian);
	}

  public abstract String getMenuItem();
  protected abstract Abstract getInstance();
  protected abstract String getDialogTitle();
  protected abstract String getPropertyNameForPort();
  protected abstract boolean isAcceptable(Info info);
}
