package org.adtpro.gui.AudioConfig;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.Mixer.Info;
import javax.sound.sampled.TargetDataLine;

import org.adtpro.resources.Messages;

public class Input extends Abstract {
    private static final long serialVersionUID = 1L;

    private static Abstract _theSingleton = null;

    /**
     * Retrieve the single instance of this class.
     * 
     * @return Log
     */
    public static Abstract getSingleton()
    {
      if (null == _theSingleton)
    	  _theSingleton = allocateSingleton();
      return _theSingleton;
    }

	protected Input() {
		super();
	}
	
	protected String getDialogTitle() {
		return Messages.getString("Gui.AudioConfig.Input.DialogTitle"); //$NON-NLS-1$
	}

	protected String getPropertyNameForPort() {
		return "AudioInputPortIndex"; //$NON-NLS-1$
	}

	protected boolean isAcceptable(final Info info) {
	  boolean result = true; 
	  try {
        final AudioFormat audioFormat = Abstract.getAudioFormat();
        final DataLine.Info dataLineInfo = new DataLine.Info(TargetDataLine.class, audioFormat);
        final Mixer mixer = AudioSystem.getMixer(info);
        final DataLine dataLine = (TargetDataLine) mixer.getLine(dataLineInfo);
		try {
			dataLine.open();
		} finally {
			dataLine.close();
		}
	  } catch (Exception e) {
    	  result = false;
      }
	  return result;
	}

	protected Abstract getInstance() {
		return _theSingleton;
	}
	
	public String getMenuItem() {
		return Messages.getString("Gui.AudioConfig.Input.MenuItem"); //$NON-NLS-1$
	}

}
