package org.adtpro.transport.audio;

import javax.sound.sampled.*;

import org.adtpro.resources.Messages;
import org.adtpro.utilities.Log;

abstract class AudioThread extends Thread {

	protected int _hardwareMixerIndex;

	protected AudioThread() {
		_hardwareMixerIndex = 0;
	}

	protected AudioThread(int hardwareMixerIndex) {
		_hardwareMixerIndex = hardwareMixerIndex;
	}

	protected Line getDataLine(String clientName, DataLine.Info info) throws LineUnavailableException {
		Line result = null;
		if (_hardwareMixerIndex == 0)
		{
			result = AudioSystem.getLine(info);
			Log.println(true, clientName + " using audio mixer "+Messages.getString("Gui.DefaultAudioMixer")+".");
		}
		else
		{
			final int hMixerIndex = _hardwareMixerIndex -1;
			final Mixer mixer = AudioSystem.getMixer(this.getMixerInfo()[hMixerIndex]);
			try
			{
				result = mixer.getLine(info);
				Log.println(true, clientName+" using audio mixer "+this.getMixerInfo()[hMixerIndex].getName()+".");
			}
			catch (Exception e)
			{
				result = AudioSystem.getLine(info);
				Log.println(true, clientName+" reverting to audio mixer "+Messages.getString("Gui.DefaultAudioMixer")+".");
			}
		}
		return result;
	}
	
	abstract protected Mixer.Info[] getMixerInfo(); 
}
